@@ -66,7 +66,7 @@ use DNS::Unbound::X ();
 our ($VERSION);
 
 BEGIN {
-    $VERSION = '0.16_04';
+    $VERSION = '0.17_01';
     XSLoader::load();
 }
 
@@ -361,8 +361,8 @@ sub set_option {
     my $err = _ub_ctx_set_option( $_[0]->{'_ub'}, "$_[1]:", $_[2] );
 
     if ($err) {
-        my $str = _ctx_err()->{$err} || "Unknown error code: $err";
-        die "Failed to set “$_[1]” ($_[2]): $str";
+        my $str = _get_error_string_from_number($err);
+        die "Failed to set “$_[1]” option ($_[2]): $str";
     }
 
     return $_[0];
@@ -378,8 +378,8 @@ sub get_option {
     my $got = _ub_ctx_get_option( $_[0]->{'_ub'}, $_[1] );
 
     if (!ref($got)) {
-        my $str = _ctx_err()->{$got} || "Unknown error code: $got";
-        die "Failed to get “$_[1]”: $str";
+        my $str = _get_error_string_from_number($got);
+        die "Failed to get “$_[1]” option: $str";
     }
 
     return $$got;
@@ -434,10 +434,50 @@ Gives the libunbound version string.
 
 #----------------------------------------------------------------------
 
+=head1 METHODS FOR ALTERING RESOLVER LOGIC
+
+The following parallel their equivalents in libunbound.
+They return I<OBJ> and throw errors on failure.
+
+=head2 I<OBJ>->hosts( $FILENAME )
+
+Z<>
+
+=head2 I<OBJ>->resolveconf( $FILENAME )
+
+Z<>
+
+=cut
+
+sub hosts {
+    my $err = _ub_ctx_hosts( $_[0]->{'_ub'}, $_[1] );
+
+    if ($err) {
+        my $str = _get_error_string_from_number($err);
+        die "Failed to set hosts file: $str";
+    }
+
+    return $_[0];
+}
+
+sub resolvconf {
+    my $err = _ub_ctx_resolvconf( $_[0]->{'_ub'}, $_[1] );
+
+    if ($err) {
+        my $str = _get_error_string_from_number($err);
+        die "Failed to set stub nameservers: $str";
+    }
+
+    return $_[0];
+}
+
+#----------------------------------------------------------------------
+
 =head1 METHODS FOR DEALING WITH ASYNCHRONOUS QUERIES
 
 Unless otherwise noted, the following methods correspond to their
-equivalents in libunbound.
+equivalents in libunbound. They return the same values as the
+libunbound equivalents.
 
 =head2 I<OBJ>->poll()
 
@@ -506,6 +546,8 @@ sub count_pending_queries {
 The following correspond to their equivalents in libunbound
 and will only work if the underlying libunbound version supports them.
 
+They return I<OBJ> and throw errors on failure.
+
 =head2 I<OBJ>->add_ta()
 
 Z<>
@@ -513,7 +555,14 @@ Z<>
 =cut
 
 sub add_ta {
-    return _ub_ctx_add_ta( $_[0]->{'_ub'}, $_[1] );
+    my $err = _ub_ctx_add_ta( $_[0]->{'_ub'}, $_[1] );
+
+    if ($err) {
+        my $str = _get_error_string_from_number($err);
+        die "Failed to add trust anchor: $str";
+    }
+
+    return $_[0];
 }
 
 =head2 I<OBJ>->add_ta_autr()
@@ -523,7 +572,14 @@ Z<>
 =cut
 
 sub add_ta_autr {
-    return _ub_ctx_add_ta_autr( $_[0]->{'_ub'}, $_[1] );
+    my $err = _ub_ctx_add_ta_autr( $_[0]->{'_ub'}, $_[1] );
+
+    if ($err) {
+        my $str = _get_error_string_from_number($err);
+        die "Failed to add managed trust anchor file: $str";
+    }
+
+    return $_[0];
 }
 
 =head2 I<OBJ>->add_ta_file()
@@ -533,7 +589,14 @@ Z<>
 =cut
 
 sub add_ta_file {
-    return _ub_ctx_add_ta_file( $_[0]->{'_ub'}, $_[1] );
+    my $err = _ub_ctx_add_ta_file( $_[0]->{'_ub'}, $_[1] );
+
+    if ($err) {
+        my $str = _get_error_string_from_number($err);
+        die "Failed to add zone-style trust anchor file: $str";
+    }
+
+    return $_[0];
 }
 
 =head2 I<OBJ>->trustedkeys()
@@ -543,7 +606,14 @@ Z<>
 =cut
 
 sub trustedkeys {
-    return _ub_ctx_trustedkeys( $_[0]->{'_ub'}, $_[1] );
+    my $err = _ub_ctx_trustedkeys( $_[0]->{'_ub'}, $_[1] );
+
+    if ($err) {
+        my $str = _get_error_string_from_number($err);
+        die "Failed to add BIND-style trust anchor file: $str";
+    }
+
+    return $_[0];
 }
 
 #----------------------------------------------------------------------
@@ -654,6 +724,16 @@ sub DESTROY {
 
 #----------------------------------------------------------------------
 
+sub _get_error_string_from_number {
+    my ($err) = @_;
+
+    if ($err) {
+        return _ctx_err()->{$err} || "Unknown error code: $err";
+    }
+
+    return undef;
+}
+
 1;
 
 =head1 LICENSE & COPYRIGHT
